TLS_PROTO(SSLv3_PROTO,   0x01, "SSLv3")
TLS_PROTO(TLSv1_0_PROTO, 0x02, "TLSv1.0")
TLS_PROTO(TLSv1_1_PROTO, 0x04, "TLSv1.1")
TLS_PROTO(TLSv1_2_PROTO, 0x08, "TLSv1.2")
TLS_PROTO(TLSv1_3_PROTO, 0x10, "TLSv1.3")
#undef TLS_PROTO
