SYSLOG_FAC(LOG_AUTH, "auth")
SYSLOG_FAC(LOG_AUTHPRIV, "authpriv")
SYSLOG_FAC(LOG_CRON, "cron")
SYSLOG_FAC(LOG_DAEMON, "daemon")
SYSLOG_FAC(LOG_FTP, "ftp")
SYSLOG_FAC(LOG_LOCAL0, "local0")
SYSLOG_FAC(LOG_LOCAL1, "local1")
SYSLOG_FAC(LOG_LOCAL2, "local2")
SYSLOG_FAC(LOG_LOCAL3, "local3")
SYSLOG_FAC(LOG_LOCAL4, "local4")
SYSLOG_FAC(LOG_LOCAL5, "local5")
SYSLOG_FAC(LOG_LOCAL6, "local6")
SYSLOG_FAC(LOG_LOCAL7, "local7")
SYSLOG_FAC(LOG_LPR, "lpr")
SYSLOG_FAC(LOG_MAIL, "mail")
SYSLOG_FAC(LOG_NEWS, "news")
SYSLOG_FAC(LOG_USER, "user")
SYSLOG_FAC(LOG_UUCP, "uucp")
